@@ -42,21 +42,27 @@ int group_count;
 char *group_names[XkbNumKbdGroups];
 char *symbol_names[XkbNumKbdGroups];
 
-#include <panel/xfce.h>
-//int default_group = 0;
 GHashTable* pGroupHash = NULL;
 
-gint get_group_count() { 
+gint 
+get_group_count() 
+{ 
   return group_count;
 }
-char *to_upper(char *src) {
+
+char*
+to_upper(char *src) 
+{
   int i = 0;
   for (i = 0; i < strlen(src); i++) {
     src[i] = toupper(src[i]);
   }
   return src;
 }
-char *to_lower(char *src) {
+
+char*
+to_lower(char *src) 
+{
   int i = 0;
   for (i = 0; i < strlen(src); i++) {
     src[i] = tolower(src[i]);
@@ -64,7 +70,9 @@ char *to_lower(char *src) {
   return src;
 }
 
-static int group_lookup(int source_value, char *from_texts[], char *to_texts[], int count) {
+static int 
+group_lookup(int source_value, char *from_texts[], char *to_texts[], int count) 
+{
   if (flexy_groups) {
     const char *source_text = from_texts[source_value];
 
@@ -86,35 +94,49 @@ static int group_lookup(int source_value, char *from_texts[], char *to_texts[],
 
 }
 
-static int group_xkb_to_res(int group_xkb_no) {
+static int 
+group_xkb_to_res(int group_xkb_no)
+{
   return group_lookup(group_xkb_no, symbol_names, group_codes, group_code_count);
 }
 
-static int group_no_res_to_xkb(int group_res_no) {
+static int 
+group_no_res_to_xkb(int group_res_no) 
+{
   return group_lookup(group_res_no, group_codes, symbol_names, group_count);
 }
 
-static char * get_group_name_by_res_no(int group_res_no) {
+static char * 
+get_group_name_by_res_no(int group_res_no) 
+{
   return group_names[group_no_res_to_xkb(group_res_no)];
 }
 
-char * get_symbol_name_by_res_no(int group_res_no) {
+char * 
+get_symbol_name_by_res_no(int group_res_no) 
+{
   return symbol_names[group_no_res_to_xkb(group_res_no)];
 }
 
-const char *get_current_group_name() {
+const char *
+get_current_group_name() 
+{
   return get_symbol_name_by_res_no(current_group_xkb_no);
 }
 
-void accomodate_group_xkb(void) {
-	XkbStateRec xkb_state;
-	XkbGetState(dsp, device_id, &xkb_state);
-	current_group_xkb_no = xkb_state.group;
+void 
+accomodate_group_xkb(void) 
+{
+  XkbStateRec xkb_state;
+  XkbGetState(dsp, device_id, &xkb_state);
+  current_group_xkb_no = xkb_state.group;
 
-	current_group_res_no = group_xkb_to_res(current_group_xkb_no);
+  current_group_res_no = group_xkb_to_res(current_group_xkb_no);
 }
 
-int do_init_xkb() {
+int 
+do_init_xkb() 
+{
   const Atom *group_source;
   Bool status;
   int major, minor, oppcode;
@@ -129,13 +151,13 @@ int do_init_xkb() {
   char *ptr1;
   int  count;
 
-  // create hash asap, so it'll be ready when events arrive
+  /* create hash asap, so it'll be ready when events arrive */
   pGroupHash = g_hash_table_new(g_direct_hash, NULL);
 
 
   /* Initialize the Xkb extension */
   status = XkbQueryExtension(dsp, &oppcode,
-    &base_event_code, &base_error_code, &major, &minor);
+      &base_event_code, &base_error_code, &major, &minor);
 
   device_id = XkbUseCoreKbd;
 
@@ -164,8 +186,8 @@ int do_init_xkb() {
     group_count = kbd_desc_ptr->ctrls->num_groups;
   } else {
     for (group_count=0;
-         group_count<XkbNumKbdGroups && group_source[group_count] != None;
-         group_count++);
+        group_count<XkbNumKbdGroups && group_source[group_count] != None;
+        group_count++);
   }
 
   if (group_count == 0) group_count=1;
@@ -207,10 +229,10 @@ int do_init_xkb() {
     }
 
     if (strncmp(ptr, "group", 5) == 0) continue;
-    if (strncmp(ptr, "inet", 4) == 0) continue;    
-    /* Filter cases like pc(pc105) (Xorg 7.0 update) */
-    if (strncmp(ptr, "pc", 2) == 0) continue;
-
+        if (strncmp(ptr, "inet", 4) == 0) continue;
+        /* Filter cases like pc(pc105) (Xorg 7.0 update) */
+        if (strncmp(ptr, "pc", 2) == 0) continue;
+        
     symbol_names[count++] = to_upper(strdup(ptr));
   }
 
@@ -239,63 +261,66 @@ int do_init_xkb() {
     }
 
     switch(group_title_source) {
-    case 1: /* Group name */
-      if (group_names[i] == NULL) {
-        char *name = get_symbol_name_by_res_no(i);
-        if (name == NULL) name = "U/A";
-        fprintf(stderr, "\nGroup Name %i is undefined, set to '%s' !\n", i+1, name);
-        group_names[i] = strdup(name);
-      }
-      break;
+      case 1: /* Group name */
+        if (group_names[i] == NULL) {
+          char *name = get_symbol_name_by_res_no(i);
+          if (name == NULL) name = "U/A";
+          fprintf(stderr, "\nGroup Name %i is undefined, set to '%s' !\n", i+1, name);
+          group_names[i] = strdup(name);
+        }
+        break;
 
-    case 2: /* Gustom name */
-      if (custom_names[i] == NULL) {
-        const char *name = get_symbol_name_by_res_no(i);
-        if (name == NULL) name = get_group_name_by_res_no(i);
-        if (name == NULL) name = "U/A";
-        fprintf(stderr, "\nCustom Name %i is undefined, set to '%s' !\n", i+1, name);
-        custom_names[i] = strdup(name);
-      }
-      break;
+      case 2: /* Gustom name */
+        if (custom_names[i] == NULL) {
+          const char *name = get_symbol_name_by_res_no(i);
+          if (name == NULL) name = get_group_name_by_res_no(i);
+          if (name == NULL) name = "U/A";
+          fprintf(stderr, "\nCustom Name %i is undefined, set to '%s' !\n", i+1, name);
+          custom_names[i] = strdup(name);
+        }
+        break;
 
-    default: /* Symbolic name (0), No title source but can be used for images (3) */
-      if (symbol_names[i] == NULL) {
-        fprintf(stderr, "\nGroup Symbol %i is undefined, set to 'U/A' !\n", i+1);
-        symbol_names[i] = strdup("U/A");
-      }
-      break;
+      default: /* Symbolic name (0), No title source but can be used for images (3) */
+        if (symbol_names[i] == NULL) {
+          fprintf(stderr, "\nGroup Symbol %i is undefined, set to 'U/A' !\n", i+1);
+          symbol_names[i] = strdup("U/A");
+        }
+        break;
     }
   }
 
   XkbGetState(dsp, device_id, &xkb_state);
   current_group_xkb_no = xkb_state.group;
 
-
- status = True;
+  status = True;
 
 HastaLaVista:
   if (kbd_desc_ptr) XkbFreeKeyboard(kbd_desc_ptr, 0, True);
   return status;
 }
 
-char *get_current_gourp_flag_name(char *filename) {
+char *
+get_current_gourp_flag_name(char *filename) 
+{
   char *group;
-  
+
   group = strdup(get_symbol_name_by_res_no(current_group_xkb_no));
   strcpy(filename, FLAGSDIR);
   strcat(filename, "/");
   strcat(filename, group);
   strcat(filename, ".png");
   to_lower(filename);
-  
+
   free(group);
-  
+
   return filename;
 }
 
 gboolean temporary_changed_display_type = FALSE;
 
-gboolean is_current_group_flag_available() {
+gboolean 
+is_current_group_flag_available() 
+{
   char filename[255];
   gboolean result = FALSE;
   GdkPixbuf *tmp = gdk_pixbuf_new_from_file(get_current_gourp_flag_name(filename), NULL);
@@ -304,17 +329,55 @@ gboolean is_current_group_flag_available() {
   return result;
 }
 
-void set_new_locale(t_xkb *ctrl) {
+char *
+xkb_get_label_markup(t_xkb *xkb)
+{
+  int font_desc;
+ 
+  if (xkb->size < 20) 
+    font_desc = 4;
+  else if (xkb->size >= 20 && xkb->size < 26)
+    font_desc = 6;
+  else if (xkb->size >= 26 && xkb->size < 32)
+    font_desc = 10;
+  else if (xkb->size >= 32 && xkb->size < 40)
+    font_desc = 12;
+  else if (xkb->size >= 40 && xkb->size < 52)
+    font_desc = 14;
+  else if (xkb->size >= 52 && xkb->size < 62)
+    font_desc = 16;
+  else if (xkb->size >= 62 && xkb->size < 70)
+    font_desc = 18;
+  else if (xkb->size >= 70 && xkb->size < 86)
+    font_desc = 20;
+  else if (xkb->size >= 86 && xkb->size < 100)
+    font_desc = 24;
+  else if (xkb->size >= 100 && xkb->size < 112)
+    font_desc = 28;
+  else if (xkb->size >= 112 && xkb->size < 124)
+    font_desc = 36;
+  else if (xkb->size >= 124)
+    font_desc = 48;
+
+  return g_markup_printf_escaped ("<span font_desc=\"%d\">%s</span>", font_desc, get_symbol_name_by_res_no (current_group_xkb_no));
+}
+
+void 
+set_new_locale(t_xkb *ctrl) 
+{
   t_xkb *plugin = (t_xkb *) ctrl;
   char filename[255];
+  char *label_markup;
   int size;
   GdkPixbuf *pixbuf, *tmp;
 
   /* Set the label   */
-  gtk_label_set_label((GtkLabel *) plugin->label, get_symbol_name_by_res_no(current_group_xkb_no));
-  
+  label_markup = xkb_get_label_markup (plugin); 
+  gtk_label_set_markup (GTK_LABEL (plugin->label), label_markup);
+  g_free(label_markup);
+
   /* Set the image */
-  size = plugin->size - 4;
+  size = 0.9 * plugin->size;
   tmp = gdk_pixbuf_new_from_file(get_current_gourp_flag_name(filename), NULL);
   if (tmp == NULL) { /* could not be loaded for some reason */
     if (plugin->display_type == IMAGE) {
@@ -324,11 +387,11 @@ void set_new_locale(t_xkb *ctrl) {
     }
   } else { /* loaded successfully */
     temporary_changed_display_type = TRUE;
-	pixbuf = gdk_pixbuf_scale_simple(tmp, size, size - (int) (size / 3), GDK_INTERP_BILINEAR);
+    pixbuf = gdk_pixbuf_scale_simple(tmp, size, size - (int) (size / 3), GDK_INTERP_BILINEAR);
     gtk_image_set_from_pixbuf((GtkImage *) plugin->image, pixbuf);
     g_object_unref(G_OBJECT(tmp));
     g_object_unref(G_OBJECT(pixbuf));
-    
+
     if (plugin->display_type == IMAGE) { 
       /* the image for the previous active layout could not be loaded */
       gtk_widget_hide(plugin->label);
@@ -339,19 +402,21 @@ void set_new_locale(t_xkb *ctrl) {
   /* Part of the image may remain visible after image or display type change */
   gtk_widget_queue_draw_area(plugin->btn, 0, 0, plugin->size, plugin->size);
 
-  // "locale per process"
-  // TBF:: bad here, it's not really a "window" related file
+  /* "locale per process" */
+  /* TBF:: bad here, it's not really a "window" related file */
   NetkWindow* win = netk_screen_get_active_window(netk_screen_get_default());
   if (pGroupHash && win)
   {
-	  gint pid = netk_window_get_pid(win);
-		printf("Storing locale %s for %d\n", get_symbol_name_by_res_no(current_group_xkb_no), pid);
-	
-		g_hash_table_insert(pGroupHash, GINT_TO_POINTER(pid), GINT_TO_POINTER(current_group_xkb_no));
+    gint pid = netk_window_get_pid(win);
+    DBG("Storing locale %s for %d\n", get_symbol_name_by_res_no(current_group_xkb_no), pid);
+
+    g_hash_table_insert(pGroupHash, GINT_TO_POINTER(pid), GINT_TO_POINTER(current_group_xkb_no));
   }
 }
 
-void handle_xevent(t_xkb *ctrl) {
+void 
+handle_xevent(t_xkb *ctrl)
+{
   XkbEvent evnt;
 
   XNextEvent(dsp, &evnt.core);
@@ -367,7 +432,9 @@ void handle_xevent(t_xkb *ctrl) {
   }
 }
 
-char * initialize_xkb(t_xkb *ctrl) {
+char * 
+initialize_xkb(t_xkb *ctrl) 
+{
   XkbEvent evnt;
   XkbStateRec state;
   int event_code, error_rtrn, major, minor, reason_rtrn;
@@ -382,20 +449,20 @@ char * initialize_xkb(t_xkb *ctrl) {
   dsp = XkbOpenDisplay(display_name, &event_code, &error_rtrn, &major, &minor, &reason_rtrn);
 
   switch (reason_rtrn) {
-  case XkbOD_BadLibraryVersion:
-    printf("Bad XKB library version.\n");
-    return NULL;
-  case XkbOD_ConnectionRefused:
-    printf("Connection to X server refused.\n");
-    return NULL;
-  case XkbOD_BadServerVersion:
-    printf("Bad X server version.\n");
-    return NULL;
-  case XkbOD_NonXkbServer:
-    printf("XKB not present.\n");
-    return NULL;
-  case XkbOD_Success:
-    break;
+    case XkbOD_BadLibraryVersion:
+      fprintf(stderr, "Bad XKB library version.\n");
+      return NULL;
+    case XkbOD_ConnectionRefused:
+      fprintf(stderr, "Connection to X server refused.\n");
+      return NULL;
+    case XkbOD_BadServerVersion:
+      fprintf(stderr, "Bad X server version.\n");
+      return NULL;
+    case XkbOD_NonXkbServer:
+      fprintf(stderr, "XKB not present.\n");
+      return NULL;
+    case XkbOD_Success:
+      break;
   }
 
   if (do_init_xkb() != True) return "N/A";
@@ -403,7 +470,7 @@ char * initialize_xkb(t_xkb *ctrl) {
   group = get_symbol_name_by_res_no(current_group_xkb_no);
 
   XkbSelectEventDetails(dsp, XkbUseCoreKbd, XkbStateNotify,
-                        XkbAllStateComponentsMask, XkbGroupStateMask);
+      XkbAllStateComponentsMask, XkbGroupStateMask);
 
   XkbGetState(dsp, device_id, &state);
   current_group_xkb_no = (current_group_xkb_no != state.group) ? state.group : current_group_xkb_no;
@@ -414,22 +481,26 @@ char * initialize_xkb(t_xkb *ctrl) {
   return group;
 }
 
-static void deinit_group_names() {
-	int i;
-	for (i=0; i< group_count; i++) {
-		if (group_names[i] != NULL) {
-		  free(group_names[i]);
-		  group_names[i] = NULL;
-		}
-		if (symbol_names[i] != NULL) {
-		  free(symbol_names[i]);
-		  symbol_names[i] = NULL;
-		}
-	}
+static void 
+deinit_group_names() 
+{
+  int i;
+  for (i=0; i< group_count; i++) {
+    if (group_names[i] != NULL) {
+      free(group_names[i]);
+      group_names[i] = NULL;
+    }
+    if (symbol_names[i] != NULL) {
+      free(symbol_names[i]);
+      symbol_names[i] = NULL;
+    }
+  }
 }
 
-void deinitialize_xkb() {
-	deinit_group_names();
+void 
+deinitialize_xkb() 
+{
+  deinit_group_names();
   XCloseDisplay(dsp);
   dsp = NULL;
 
@@ -437,37 +508,49 @@ void deinitialize_xkb() {
   pGroupHash = NULL;
 }
 
-int get_connection_number() {
+int 
+get_connection_number()
+{
   return ConnectionNumber(dsp);
 }
 
 /* Sets the kb layout to the next layout */
-int do_change_group(int increment, t_xkb *ctrl) {
+int 
+do_change_group(int increment, t_xkb *ctrl) 
+{
   if (group_count <= 1) return 0;
   XkbLockGroup(dsp, device_id,
-    (current_group_xkb_no + group_count + increment) % group_count); // why not simply (current_group_xkb_no + increment) % group_count ?
+      (current_group_xkb_no + group_count + increment) % group_count); 
+      /* why not simply (current_group_xkb_no + increment) % group_count ? */
   handle_xevent(ctrl);
   return 1;
 }
 
-int do_set_group(int group, t_xkb *ctrl) {
+int 
+do_set_group(int group, t_xkb *ctrl) 
+{
   if (group >= group_count)
     return 0;
+  
   XkbLockGroup(dsp, device_id, group);
-	accomodate_group_xkb();
-	set_new_locale(ctrl);
-	
+  accomodate_group_xkb();
+  set_new_locale(ctrl);
+
   return 1;
 }
 
-gboolean gio_callback(GIOChannel *source, GIOCondition condition, gpointer data) {
+gboolean 
+gio_callback(GIOChannel *source, GIOCondition condition, gpointer data) 
+{
   handle_xevent((t_xkb *) data);
   return TRUE;
 }
 
-void react_active_window_changed(gint pid, t_xkb *ctrl)
+void 
+react_active_window_changed(gint pid, t_xkb *ctrl)
 {
-  if (ctrl->enable_perapp) {
+  if (ctrl->enable_perapp)
+  {
     gpointer pKey=0, pVal=0;
     gint new_group_xkb_no = ctrl->default_group;
 
@@ -478,9 +561,12 @@ void react_active_window_changed(gint pid, t_xkb *ctrl)
   }
 }
 
-void react_application_closed(gint pid)
+/* TODO: destroy hashtable if perapp layout is disabled? */
+void 
+react_application_closed(gint pid)
 {
   if (pGroupHash) {
     g_hash_table_remove(pGroupHash, GINT_TO_POINTER(pid));
   }
 }
+
